@@ -352,7 +352,8 @@ pub fn map_parasite_s2g(para_as_species: &mut ArenaTree<String>,
     }
 }
 
-pub fn map_gene_host(gene_trees: &mut Vec<ArenaTree<String>>, tree_para_pipe: &mut ArenaTree<String>, tree_host_pipe:&mut ArenaTree<String>) {
+/// Map gene trees to  host tree
+pub fn map_gene_host(gene_trees: &mut Vec<ArenaTree<String>>, tree_para_pipes:  &mut Vec<ArenaTree<String>>, tree_host_pipe:&mut ArenaTree<String>) {
     // unmap the host tree
     let mut j = 0;
     let nb_nodes = tree_host_pipe.arena.len();
@@ -363,62 +364,80 @@ pub fn map_gene_host(gene_trees: &mut Vec<ArenaTree<String>>, tree_para_pipe: &m
         tree_host_pipe.arena[j].height =  PIPEBLOCK;
         j = j + 1 ;
     }
-
-
     let mut i = 0;
     let nb_genes = gene_trees.len();
+    let nb_par = tree_para_pipes.len();
     while i < nb_genes {
-        // let  mut gene_tree = &gene_trees[i];
-        // for &mut node in  gene_tree {
-        //     println!("debug NODE {:?}",&node);
         let mut j = 0;
         let nb_nodes = gene_trees[i].arena.len();
-        let mut amodifier:std::vec::Vec<(usize,String)> = Vec::new(); // a cause des noeuds de
+        // let mut amodifier:std::vec::Vec<(usize,String)> = Vec::new(); // a cause des noeuds de
                                                                       // de genes ajoutes pour
                                                                       // matcher avec l'arbre
                                                                       // d'espèces
+        let mut virtual_nodes:std::vec::Vec<usize> = Vec::new();
         while j < nb_nodes {
-            // println!("debug NODE {:?}",gene_trees[i].arena[j].location);
-            // cherche l'espce dans l'arbre du parasite
-            let espece = gene_trees[i].arena[j].location.clone();
-            let espece_index = tree_para_pipe.get_index(espece);
-            let espece_index = match espece_index {
-                Ok(n) => n,
-                Err(_e) => {
-                    panic!("[map_gene_host] Unable to match species {} in parasite tree",
-                    gene_trees[i].arena[j].location.clone());
-                },
-            };
-            // println!("debug ESPECE {:?}",espece_index);
-            // println!("debug ESPECE {:?}",tree_para_pipe.arena[espece_index]);
-            gene_trees[i].arena[j].location =  tree_para_pipe.arena[espece_index].location.clone();
-            if  tree_para_pipe.arena[espece_index].is_a_transfert {
-                gene_trees[i].arena[j].is_a_transfert = true;
-                let parent = gene_trees[i].arena[j].parent;
-                match parent {
-                    None => panic!("[map_gene_host] Node has no parent"),
-                    Some(p) => gene_trees[i].arena[p].e = Event::BifurcationOut,
+            // cherche l'espce dans les arbres du parasite
+            let mut global_find_spec = false;
+            let  mut k = 0;
+            while k < nb_par {
+                let espece = gene_trees[i].arena[j].location.clone();
+                let espece_index = tree_para_pipes[k].get_index(espece);
+                let mut find_spec = false;
+                let espece_index = match espece_index {
+                    Ok(n) => {
+                        if find_spec { panic!("[map_gene_host] Multiple species matches") }
+                        find_spec = true;
+                        global_find_spec = true;
+                        n
+                    },
+                    Err(_e) => {
+                        info!("species {} not found in parasite tree number {}", gene_trees[i].arena[j].location.clone(),k);
+                        0
+                    },
                 };
-            }
-            if  gene_trees[i].arena[j].e == Event::Loss {
-                let species_parent = tree_para_pipe.arena[espece_index].parent;
-                match species_parent {
-                    None => panic!("[map_gene_host] Node has no parent"),
-                    Some(sp) => {
-                        gene_trees[i].arena[j].location =  tree_para_pipe.arena[sp].location.clone();
+                // location of gene node is found in parasite tree
+                if find_spec {
+                    // println!("debug AVANT {:?}",gene_trees[i].arena[j]);
+                    gene_trees[i].arena[j].location =  tree_para_pipes[k].arena[espece_index].location.clone();
+                    // println!("debug APRES {:?}",gene_trees[i].arena[j]);
+                    if  tree_para_pipes[k].arena[espece_index].is_a_transfert {
+                        gene_trees[i].arena[j].is_a_transfert = true;
                         let parent = gene_trees[i].arena[j].parent;
                         match parent {
                             None => panic!("[map_gene_host] Node has no parent"),
-                            Some(p) => {
-                                amodifier.push((p,tree_para_pipe.arena[sp].location.clone()));
-                                let fils = &gene_trees[i].arena[p].children;
-                                amodifier.push((fils[0],tree_para_pipe.arena[sp].location.clone()));
-                                amodifier.push((fils[1],tree_para_pipe.arena[sp].location.clone()));
-                            },
+                            Some(p) => gene_trees[i].arena[p].e = Event::BifurcationOut,
                         };
-                    },
-                };
+                    }
+                    // if  gene_trees[i].arena[j].e == Event::Loss {
+                    //     let species_parent = tree_para_pipes[k].arena[espece_index].parent;
+                    //     match species_parent {
+                    //         None => panic!("[map_gene_host] Node has no parent"),
+                    //         Some(sp) => {
+                    //             // gene_trees[i].arena[j].location =  tree_para_pipes[k].arena[sp].location.clone();
+                    //             let parent = gene_trees[i].arena[j].parent;
+                    //             match parent {
+                    //                 None => panic!("[map_gene_host] Node has no parent"),
+                    //                 Some(p) => {
+                    //                     amodifier.push((p,tree_para_pipes[k].arena[sp].location.clone()));
+                    //                     let fils = &gene_trees[i].arena[p].children;
+                    //                     amodifier.push((fils[0],tree_para_pipes[k].arena[sp].location.clone()));
+                    //                     amodifier.push((fils[1],tree_para_pipes[k].arena[sp].location.clone()));
+                    //                 },
+                    //             };
+                    //         },
+                    //     };
+                    // }
 
+                    if  gene_trees[i].arena[j].e == Event::Undef {
+                        if gene_trees[i].arena[j].children.len() > 0 {
+                            virtual_nodes.push(j);
+                        }
+                    }
+                }
+                k = k + 1;
+            }
+            if !global_find_spec {
+                panic!("[map_gene_host] Unable to map node {:?} loc={:?}",gene_trees[i].arena[j].name.clone(),gene_trees[i].arena[j].location.clone())
             }
             // On s'attend pas a des files de speciation de gene dans un meme tuyeau -> bug d'Affichage
             // Solution pourrie : remplacer les spec par des dupl
@@ -429,19 +448,28 @@ pub fn map_gene_host(gene_trees: &mut Vec<ArenaTree<String>>, tree_para_pipe: &m
             // gene_trees[i].arena[j].e =  tree_para_pipe.arena[espece_index].e;
             j = j + 1;
         }
-        for (j,spec) in amodifier {
-            gene_trees[i].arena[j].location = spec;
+        for j in virtual_nodes {
+            let espece =  gene_trees[i].arena[j].location.clone();
+            let espece_index = tree_host_pipe.get_index(espece);
+            let espece_index = match espece_index {
+                Ok(n) => {
+                    n
+                },
+                Err(_e) => {
+                    panic!("[map_gene_host] species not found in host tree");
+                },
+            };
+            let parent =  tree_host_pipe.arena[espece_index].parent;
+            let parent = match parent {
+                Some(p) => p,
+                None => panic!("[map_gene_host] species has no parent in host tree"),
+            };
+            let espece = &tree_host_pipe.arena[parent].name;
+            gene_trees[i].arena[j].location = espece.to_string();
         }
-        // for index in &mut gene_trees[i].arena {
-        //      println!("debug NODE {:?} {:?}",&index.idx,&index.location);
-        //      let idx = &index.idx;
-        //      // &index.location = "lolo".to_string();
-        //      gene_trees[i].arena[*idx].location =  "lolo".to_string();
-        // }
-        // }
         i = i + 1;
-        // for &mut node in  &gene_trees[i]
     }
+    // println!("DEBUG  [map_gene_host] FINAL HOST = {:?}",tree_host_pipe)
 }
 
 //  get the trasnsfers in a gene tree
